function [M1, M2, objf] = MVMC(omega, S_ob, Z1,Z2, params)
%% Multi-view clustering by MVMC (for two views)
% Input:
%     omega: index of side information point
%     S_ob: the similarity matrix with side information (n*n)
%     Z1: first k left singular vectors of feature of view 1
%     Z2: first k left singular vectors of feature of view 2
%     params: parameters used for parameter selection 
%          -- 'C1': regularization parameter C1 in Eq. (4)
%          -- 'C2': regularization parameter C2 in Eq. (4)
%          -- 'eta1': step parameter eta1 in Eq. (11) by default 1e-4
%          -- 'eta2': step parameter eta1 in Eq. (11) by default 1e-4
%          -- 'numIter': iteration number of projected gradient descend
% Output:
%     M1: subspace of view 1
%     M2: subspace of view 2
%     objf: values of objective functions for recording performance
%          -- 'obj': the value of whole objective 
%          -- 'obj_Trace': the value of trace term 
%          -- 'obj_Loss_1': the value of loss on given side information 
%          -- 'obj_Loss_2': the value of loss on all index 
% Developed with Matlab R2014b by Peng Zhao (zhaop@lamda.nju.edu.cn)
% Reference: P. Zhao, Y. Jiang, and Z.-H. Zhou. Multi-view matrix completion for 
% clustering with side information. In: Proceedings of the 21st Pacific-Asia 
% Conference on Knowledge Discovery and Data Mining (PAKDD'17), LNAI, Jeju, Korea, 2017.

    C1 = params.C1;
    C2 = params.C2;
    eta1 = params.eta1;
    eta2 = params.eta2;
    numIter = params.numIter;
    max_iter = 10;      % iteration rounds of iterative optimization
    % for recording the value objective function
    obj_Trace=zeros(max_iter,1); 
    obj_Loss_1=zeros(max_iter,1); 
    obj_Loss_2=zeros(max_iter,1);
    obj=zeros(max_iter,1);
    %% Step 1. Initialization
    [~,k]=size(Z1);
    Sigma1=zeros(k,k);
    Sigma2=zeros(k,k);
    S_avg = S_ob;
    M1 = Z1'*S_avg*Z1; 
    M2 = Z2'*S_avg*Z2; 
    FnormUpper=k^2/10;
    
    for iter = 1:max_iter
    %% Step 2. Minimizing object over M1 and M2 with fixed S
        for t=1:numIter
            derF_M_1 = C2*(Z1'*(Z1*M1*Z1'-S_avg)*Z1 + C1*Z1'*(omega.*(Z1*M1*Z1'-S_ob))*Z1) + eye(k);
            derF_M_2 = C2*(Z2'*(Z2*M2*Z2'-S_avg)*Z2 + C1*Z2'*(omega.*(Z2*M2*Z2'-S_ob))*Z2) + eye(k);
            M1 = M1-eta1*derF_M_1;
            M2 = M2-eta2*derF_M_2;

            if norm(M1,'fro')>FnormUpper || norm(M2,'fro')>FnormUpper
                M1 = M1*FnormUpper/norm(M1,'fro');
                M2 = M2*FnormUpper/norm(M2,'fro');           
            end
            [U1,Sigma1] = eig(M1);
            [U2,Sigma2] = eig(M2);
            % projecting M1, M2 back to semi-definite positive cone
            Sig1=diag(Sigma1); Sig2=diag(Sigma2); 
            M1 = U1*diag(max(Sig1,0))*U1';
            M2 = U2*diag(max(Sig2,0))*U2';
        end
    %% Step 3. Minimizing object over S with fixed M1 and M2      
        S1 = Z1*M1*Z1'; S2 = Z2*M2*Z2';
        S_avg = (S1 + S2)/2;
        S_avg(S_avg>1) = 1; S_avg(S_avg<0) = 0;
    %% Step 4. Record the value objective function in each iteration
        loss1_val = 0.5*C1*(norm((omega.*(S1-S_ob)),'fro')^2+norm((omega.*(S2-S_ob)),'fro')^2);
        loss2_val = 0.5*C2*(norm(S_avg-S1,'fro')^2+norm(S_avg-S2,'fro')^2);
        Trace_val = sum(diag(Sigma1))+sum(diag(Sigma2));
        obj_val = Trace_val + loss1_val + loss2_val;
        obj_Trace(iter,:) = Trace_val; 
        obj_Loss_1(iter,:) = loss1_val; 
        obj_Loss_2(iter,:) = loss2_val;
        obj(iter,:) = obj_val;
    end
    objf.obj = obj;
    objf.obj_Trace = obj_Trace;
    objf.obj_Loss_1 = obj_Loss_1;
    objf.obj_Loss_2 = obj_Loss_2;
end
